# BEGIN COPYRIGHT
# *************************************************************************
# Licensed Materials - Property of IBM 
# 5724-L01, 5655-N53, 5724-I82, 5655-R15
# Copyright IBM Corporation 2006, 2012. All rights reserved. 
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
# *************************************************************************
# END COPYRIGHT
from java.lang import System, String
import sys

print "*************************************************************************************"
print "start: init"
print ""

fileSeparator = System.getProperty("file.separator");
configUtilsScript = System.getProperty("was.install.root") + fileSeparator + "util" + fileSeparator + "config" + fileSeparator + "admin" + fileSeparator + "configUtils.py"
print "load configuration utils from ", configUtilsScript
execfile(configUtilsScript);

print ""
print "end: init"

#--------------------------------------------------------------------------
# get the directory where WAS is installed
#--------------------------------------------------------------------------
def getWASHome():
	return getPropertyValue('was.install.root')
	

#------------------------------------------------------------------------------
# get the property value by the the property name; If property not available, return ''
#------------------------------------------------------------------------------

def getPropertyValue(propertyName):
	propertyValue = java.lang.System.getProperty(propertyName)
	if(propertyValue == None):
		return ''
	return propertyValue
	
#------------------------------------------------------------------------------
# a utility method
#------------------------------------------------------------------------------

def wsadminToList(inStr):
  outList=[]
  if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
    inStr = inStr[1:-1]
    tmpList = inStr.split(" ")
  else:
    tmpList = inStr.split("\n")  #splits for Windows or Linux
  for item in tmpList:
    item = item.rstrip();        #removes any Windows "\r"
    if (len(item)>0):
      outList.append(item)
  return outList
#endDef

#------------------------------------------------------------------------------
# check whether the specified application is installed on specified cell
#------------------------------------------------------------------------------
def checkApplicationExist(applicationName):
	exist = 0
	applications = wsadminToList(AdminApp.list())
	for application in applications:
		if (application == applicationName):
			exist = 1
			return exist
	return exist
#------------------------------------------------------------------------------
# uninstall the application with the specified parameter
#------------------------------------------------------------------------------
def uninstallApplication(applicationName, node, server):
	result = 0
	undeployOptions = ' -node ' + node + ' -server ' + server
	try:
		AdminApp.uninstall(applicationName,undeployOptions)
	except:
		print "Error happened when trying to uninstall the application: " + applicationName
		result = 1
	AdminConfig.save()
	return result

#------------------------------------------------------------------------------
# install the application with the specified parameter
#------------------------------------------------------------------------------
def installApplication(applicationName, earLocation, cell, node, server):
	result = 0
	whetherApplicationExist = checkApplicationExist(applicationName)
	
	#1.IF: the application was not installed yet, install the application 
	if(whetherApplicationExist == 0):
		deployOptions = '-appname ' + applicationName + ' -cell ' +  cell + ' -node ' + node + ' -server ' + server
		print "deployOptions :"  + deployOptions
		try:
			AdminApp.install(earLocation, deployOptions)
		except:
			print "exception happened when trying to install application: " + applicationName + " with deployOptions " + deployOptions
			result = 1
		else:
			AdminConfig.save()
			
	#2.Else: the application was installed, add the mapping if available
	else:
		deployOptions = ' -operation update -contents ' + earLocation + ' -appname ' + applicationName + ' -cell ' + cell + ' -node ' + node  + ' -server ' + server
		print "deployOptions :"  + deployOptions
		try:
			AdminApp.update(applicationName, 'app' , deployOptions)
			print "Update done for EAR with applicationName: " + applicationName + " deployOptions:"  + deployOptions
		except:
			print "exception happened when trying to update application: " + applicationName + " with deployOptions " + deployOptions
			result = 1
			return result
		else:
			AdminConfig.save()

		#2.1 Construct the new target
		newTarget = 'WebSphere:cell=' + cell + ',node=' + node + ',server=' + server
		print "newTarget :"  + newTarget
		#2.2 Get the old target information
		moduleList = AdminApp.listModules(applicationName, '-server')
		infoList = moduleList.split('#')
		module = 'RemoteAL'
		uri = infoList[1].replace('+',',')
		oldTarget = infoList[2]
		
		#2.3 check whether the target already exist in the target mapping
		result = checkTargetExist(oldTarget,newTarget)
		
		#2.3.1 If the target map is already defined, do nothing	
		if(result == 0):
			print "the application :" + applicationName + " is already mapped to " + newTarget
		
		#2.3.2 Otherwise, add the target mapping to the server
		else:
			#the moduleList is in format like 
			try:
				detailOption = [[module,uri,oldTarget + '+' + newTarget]]
				options = ['-MapModulesToServers',detailOption]
				AdminApp.edit(applicationName, options)
			except:
				print "exception happened when trying to add the target mapping: " + newTarget
				result = 1
			else:
				AdminConfig.save()
		
	return result;
	
#------------------------------------------------------------------------------
# remove the mapping 
#------------------------------------------------------------------------------
def deleteOldMapping(oldAppName,cell,node,server,cluster):
	if(checkApplicationExist(oldAppName)==0):
		return
	whetherOldMappingExist = 0
	newTargetList = ''
	moduleList = AdminApp.listModules(oldAppName,'-server')
	infoList = moduleList.split('#')
	module = 'RemoteAL'
	uri = infoList[1].replace('+',',')
	oldTarget = infoList[2]
	oldTargetList = oldTarget.split('+')
	if(cluster ==None):
		oldTarget =  'WebSphere:cell=' + cell + ',node=' + node + ',server=' + server
	else:
		oldTarget = 'WebSphere:cell=' + cell + ',cluster=' + cluster
	for target in oldTargetList:
		if (target  == oldTarget):
			oldTargetList.remove(oldTarget)
			whetherOldMappingExist =1
			break
	if(whetherOldMappingExist==0):
		return
	if(oldTargetList==[]):
		try:
			AdminApp.uninstall(oldAppName)
		except:
			print "Error happened when trying to uninstall the application: " + applicationName
		else:
			AdminConfig.save()
			return
	sep = '+'
	newTarget = sep.join(oldTargetList)
	try:
		detailOption = [[module,uri,newTarget]]
		options = ['-MapModulesToServers',detailOption]
		AdminApp.edit(oldAppName, options)
	except:
		print "exception happened when trying to edit target mapping"
	else:
		AdminConfig.save()
#------------------------------------------------------------------------------
# install the application with the specified parameter
#------------------------------------------------------------------------------
def installApplicationCluster(applicationName, earLocation, cell, cluster):
	result = 0
	whetherApplicationExist = checkApplicationExist(applicationName)
	
	#1.IF: the application was not installed yet, install the application 
	if(whetherApplicationExist == 0):
		deployOptions = '-appname ' + applicationName + ' -cell ' +  cell + ' -cluster ' + cluster
		print "deployOptions :"  + deployOptions
		try:
			AdminApp.install(earLocation, deployOptions)
		except:
			print "exception happened when trying to install application: " + applicationName + " with deployOptions " + deployOptions
			result = 1
		else:
			AdminConfig.save()
			
	#2.Else: the application was installed, add the mapping if available
	else:
		deployOptions = ' -operation update -contents ' + earLocation + ' -appname ' + applicationName + ' -cell ' + cell + ' -cluster ' + cluster
		print "deployOptions :"  + deployOptions
		try:
			AdminApp.update(applicationName, 'app', deployOptions)
			print "Update done for EAR with applicationName: " + applicationName + " deployOptions:"  + deployOptions
		except:
			print "exception happened when trying to update application: " + applicationName + " with deployOptions " + deployOptions
			result = 1
			return result
		else:
			AdminConfig.save()

		# 2.1 Construct the new target
		newTarget = 'WebSphere:cell=' + cell + ',cluster=' + cluster 
		
		print "newTarget :"  + newTarget
		# 2.2 Get the old target information
		moduleList = AdminApp.listModules(applicationName, '-server')
		infoList = moduleList.split('#')
		module = 'RemoteAL'
		uri = infoList[1].replace('+',',')
		oldTarget = infoList[2]
		
		#2.3 check whether the target already exist in the target mapping
		result = checkTargetExist(oldTarget,newTarget)
		
		#2.3.1 If the target map is already defined, do nothing	
		if(result == 0):
			print "the application :" + applicationName + " is already mapped to " + newTarget
		
		#2.3.2 Otherwise, add the target mapping to the server
		else:
			try:
				detailOption = [[module,uri,oldTarget + '+' + newTarget]]
				options = ['-MapModulesToServers',detailOption]
				AdminApp.edit(applicationName, options)
			except:
				print "exception happened when trying to add the target mapping: " + newTarget
				result = 1
			else:
				AdminConfig.save()
		
	return result;
#------------------------------------------------------------------------------
# check whether the newTarget already exist in oldTarget;If true,return 0; otherwise, return 1
#------------------------------------------------------------------------------
def checkTargetExist(oldTarget,newTarget):
	result = 1
	targetList = oldTarget.split('+')
	for target in targetList:
		if(target == newTarget):
			result = 0
	return result
#------------------------------------------------------------------------------
# get the current cell
#------------------------------------------------------------------------------
def getCell():
	cellName = '';
	cellList = wsadminToList(AdminConfig.list('Cell'))
	for cell in cellList:
		cellName = AdminConfig.showAttribute(cell,'name')
	return cellName


#------------------------------------------------------------------------------
# get the nodes defined in specified cell
#------------------------------------------------------------------------------
def getNodeList(cell):
	nodeNameList = []
	cellId = AdminConfig.getid('/Cell:' + cell)
	if(cellId == ''):
		return nodeNameList
	nodeList = wsadminToList(AdminConfig.list('Node',cellId))
	for node in nodeList:
		nodeName = AdminConfig.showAttribute(node,'name')
		nodeNameList.append(nodeName)
	return nodeNameList

#------------------------------------------------------------------------------
# get the servers defined in specified cell and node; need to filter out the node agent, cluster member
#------------------------------------------------------------------------------
def getServerList(cell,node):
	serverNameList = []
	nodeId = AdminConfig.getid('/Cell:' + cell + '/Node:' + node)
	if(nodeId == ''):
		return serverNameList
	serverList = wsadminToList(AdminConfig.list('Server',nodeId))
	for server in serverList:
		serverName = AdminConfig.showAttribute(server,'name')
		serverType = AdminConfig.showAttribute(server,'serverType')		
		clusterName = AdminConfig.showAttribute(server,'clusterName')
		if(clusterName == None):
			if(serverName != 'nodeagent'):
				serverNameList.append(serverName)
	return serverNameList
#------------------------------------------------------------------------------
# get the location of installable RemoteAL application
#------------------------------------------------------------------------------	
def getRemoteALApplicationLocation(cell,node):
	return getWASHome() + '/installableApps/RemoteAL.ear'

#------------------------------------------------------------------------------
# check whether the update is required for current node; If true, return 0; Otherwise, return 1
# 1. If the node is dmgr node, do nothing. The installation of RemoteAL application is done for the first custom node with server specified
# 
# 2. 
#------------------------------------------------------------------------------
def checkWhetherUpdateRequired():
	result = 1
	nodeType = getPropertyValue('wbi.upgrade.node.type')
	nodeVersion = getPropertyValue('wbi.upgrade.previous.version')
	updatedCluster = getUpdatedCluster()
	
	if(updatedCluster != ''):
		result = 0
	elif (nodeType == 'managed') or (nodeType == 'unmanaged'):
		if(nodeVersion != ''):
			result = 0
	
	return result

# the procedure to remove the application mapping on web servers
def deleteWebServerMapping(oldAppName):
	if (checkApplicationExist(oldAppName)==0):
		print infoPrefix + "No deployment info for module: " + oldAppName
		return

	moduleList = AdminApp.listModules(oldAppName,'-server')
	infoList = moduleList.split('#')
	ralModule = 'RemoteAL'
	uri = infoList[1].replace('+',',')
	print "uri: "+uri
	oldTarget = infoList[2]
	print "oldTarget: "+oldTarget
	oldTargetList = oldTarget.split('+')
	newTargetList = []
	for target in oldTargetList:
		if(isWebServerTarget(target)==0):
			newTargetList.append(target)
	
	sep ='+'
	newTarget = sep.join(newTargetList)
	print "newTarget: "+newTarget
	try:
		detailOption = [[ralModule,uri,newTarget]]
		print infoPrefix + "detailOption: "
		print detailOption
		options = ['-MapModulesToServers',detailOption]
		print infoPrefix + "new map options after removing web server is "
		print options
		AdminApp.edit(oldAppName, options)
	except:
		print "exception happened when trying to edit target mapping"
	else:
		AdminConfig.save()

def isWebServerTarget(target):
	# oldTarget =  'WebSphere:cell=' + cell + ',node=' + node + ',server=' + server.
	# example: WebSphere:cell=localhostNode01Cell,node=nlNode01,server=server1
	#          WebSphere:cell=localhostNode01Cell,node=tedt60-node,server=web1
	if (target.find('cluster=')!=-1):
		print infoPrefix + "A cluster target: "+target
		return 0
	targetUnits = target.split(',')
	cellInfo = targetUnits[0]
	nodeInfo = targetUnits[1]
	serverInfo = targetUnits[2]
	cellName = cellInfo[15:len(cellInfo)]
	nodeName = nodeInfo[5:len(nodeInfo)]
	serverName = serverInfo[7:len(serverInfo)]
	nodeId = AdminConfig.getid('/Cell:'+cellName+'/Node:'+nodeName)
	entriesStr = AdminConfig.list('ServerEntry',nodeId)
	entries = wsadminToList(entriesStr)
	for entry in entries:
		currentServerName = AdminConfig.showAttribute(entry,'serverName')
		if(currentServerName==serverName):
			serverType = AdminConfig.showAttribute(entry,'serverType')
			if(serverType=="WEB_SERVER"):
				print infoPrefix + "web server target for "+target
				return 1
	
	return 0
	
#------------------------------------------------------------------------------
# get the current node name which is migrated
#------------------------------------------------------------------------------
def getUpdatedNode():
	#TODO
	return getPropertyValue('local.node')

#------------------------------------------------------------------------------
# get the current cluster name which is migrated, if not available return ''
#------------------------------------------------------------------------------
def getUpdatedCluster():
	return getPropertyValue('wbi.upgrade.cluster.name')

#------------------------------------------------------------------------------
# Create RAL Security Authentication Alias
#------------------------------------------------------------------------------
def createRALUserAuthAlias():
    global adminUserName, adminPassword;

    print ""
    print "Start of Create RAL Auth Alias "
    print ""


    createAuthAlias("RALSecurity_Auth_Alias", adminUserName, adminPassword, "Authentication Alias for RAL User" );

    print ""
    print "End of Create RAL Auth Alias "
    print ""

#endDef

#------------------------------------------------------------------------------
# Main entry point
# 1. Read the properties to decide whether RemoteAL application need to updated
#    
#
#
#------------------------------------------------------------------------------

infoPrefix = "RemoteAL.INFO: "
errorPrefix = "RemoteAL.ERROR: "
print infoPrefix + "RemoteAL migration started"

#0 Remove the web server target mapping first on dmgr and un-managed node
nodeType = getPropertyValue('wbi.upgrade.node.type')
print infoPrefix + "the current update node is: " + getUpdatedNode()
print infoPrefix + "the current update node type is: " + nodeType
if (nodeType == 'dmgr') or (nodeType == 'unmanaged'):
	deleteWebServerMapping('RemoteAL')

# 1. check whether the update procedure should be invoked
whetherUpdateRequired = checkWhetherUpdateRequired()

# 2. If update required, perform update
if (whetherUpdateRequired == 0):	
	#2.1 Initialization
    previousRemoteALApplicationName = 'RemoteAL'
    remoteALApplicationName = 'RemoteAL61'
    cell = getCell()
    node = getUpdatedNode()
    remoteALLocation = getRemoteALApplicationLocation(cell,node)
    print infoPrefix + "Remote AL Location: " + remoteALLocation
    cluster = getUpdatedCluster()
    print infoPrefix + "cluster: " + cluster
    if (cluster == ''):
		serverList = getServerList(cell,node)
		for server in serverList:
			print infoPrefix + "server: " + server
			deleteOldMapping(previousRemoteALApplicationName,cell,node,server,None)
			installApplication(remoteALApplicationName,remoteALLocation,cell,node,server) 
    else:
		deleteOldMapping(previousRemoteALApplicationName,cell,None,None,cluster)
		installApplicationCluster(remoteALApplicationName,remoteALLocation,cell,cluster)
	
# 3. Otherwise update not required, do nothing
else:
	print "Nothing need to be done for AL component"

# 4. Check for Authentication Alias.
#System.setProperty("bpm.upgrade.username", "admin");
try:
        cellType = getCellType();
        adminUserName = str(System.getProperty("bpm.upgrade.username"));
        print "The value of adminUserName is " + adminUserName;
        adminPassword = str(System.getProperty("bpm.upgrade.password"));

        if (isEmptyString(adminUserName) == "True"):
                adminUserName = "admin";
                print "The value of adminUserName is set to default ", adminUserName

        if (isEmptyString(adminPassword) == "True"):
                adminPassword = "admin";
                print "The value of adminPassword is set to default value"

        if (cellType == "STANDALONE"):
                print "Configuring stand-alone profile."
                print ""

                createRALUserAuthAlias();
                AdminConfig.save();

                print ""
                print "*************************************************************************************"
                print "The RAL Authenticatio Alias Created."
                print "*************************************************************************************"
                print ""

        elif (cellType == "DISTRIBUTED"):
                print "Updating the distributed (dmgr or customized) profile"
                # check whether the local node is a deployment manager Node
                isDMgrNode = isDMgrNode();

                if (isDMgrNode != "False"):
                        print "Updating the distributed dmgr profile"
                        
                        createRALUserAuthAlias();
                        AdminConfig.save();

                print ""
                print "*************************************************************************************"
                print "The RAL Authenticatio Alias Created."
                print "*************************************************************************************"
                print ""

        else:
                print "The distributed custom profile does not need an update."
except:
        print ""
        print "*************************************************************************************"
        print " The RAL Authenticatio Alias could not be created. The exception type is: ", sys.exc_type, ". The exception description is: ", sys.exc_value
        print "*************************************************************************************"
        print ""
        # rethrow the exception so the calling Ant script fails and eventually results in INSTCONFFAILED instead of INSTCONFSUCCESS
        raise



